# Generated by Django 3.0.3 on 2020-02-08 13:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0079_auto_20200206_0600'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='messages',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(condition=models.Q(guid=None), fields=('checksum', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='messages',
            constraint=models.UniqueConstraint(fields=('checksum', 'guid', 'state', 'theme', 'to_whom', 'user'), name='Messages_unique_constraint_1'),
        ),
    ]
