# Generated by Django 2.2.5 on 2019-09-16 17:59

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0017_auto_20190913_1056'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chat_users',
            name='props',
            field=bitfield.models.BitField((('compulsory_reading', 'Обязательное прочтение'), ('relevant', 'Актуальность'), ('readonly', 'Только чтение')), db_index=True, default=0),
        ),
    ]
