# Generated by Django 2.2.5 on 2019-09-05 05:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0058_auto_20190905_0517'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='messages',
            options={'ordering': ('date_create',), 'verbose_name': 'Сообщения'},
        ),
    ]
