# Generated by Django 2.2.5 on 2019-09-09 13:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0014_auto_20190909_1042'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='chat_usersview',
            table='twits_chat_users_view',
        ),
    ]
