# Generated by Django 2.2.5 on 2019-09-08 06:10

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0061_auto_20190907_0443'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages_files_refs',
            name='message',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages'),
        ),
        migrations.AlterField(
            model_name='messages_files_refs',
            name='messages_file',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_files'),
        ),
    ]
