# Generated by Django 2.2.7 on 2019-11-16 12:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0007_bannedip'),
    ]

    operations = [
        migrations.DeleteModel(
            name='BannedIP',
        ),
        migrations.DeleteModel(
            name='UntrackedUserAgent',
        ),
    ]
