# Generated by Django 2.2.4 on 2019-08-21 17:21

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0017_holidays_day_type'),
    ]

    operations = [
        migrations.RenameField(
            model_name='holidays',
            old_name='day_type',
            new_name='daytype',
        ),
    ]
