# Generated by Django 3.0.2 on 2020-01-22 10:29

from django.db import migrations, models
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0002_jasper_reports_users'),
    ]

    operations = [
        migrations.AddField(
            model_name='jasper_reports',
            name='host',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
        migrations.AddField(
            model_name='jasper_reports',
            name='password',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
        migrations.AddField(
            model_name='jasper_reports',
            name='port',
            field=models.PositiveIntegerField(default=None),
        ),
        migrations.AddField(
            model_name='jasper_reports',
            name='url',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
        migrations.AddField(
            model_name='jasper_reports',
            name='username',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
    ]
