# Generated by Django 2.2.4 on 2019-08-21 20:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0018_auto_20190821_1721'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Holidays_view',
        ),
    ]
